# autogenerated module
from typing import ClassVar

from ..core import resource as res
from ..models import admissionregistration_v1alpha1 as m_admissionregistration_v1alpha1


class ValidatingAdmissionPolicyStatus(res.GlobalSubResource, m_admissionregistration_v1alpha1.ValidatingAdmissionPolicy):
    """* **Extends**: ``models.admissionregistration_v1alpha1.ValidatingAdmissionPolicy``
       * **Type**: Global Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('admissionregistration.k8s.io', 'v1alpha1', 'ValidatingAdmissionPolicy'),
        parent=res.ResourceDef('admissionregistration.k8s.io', 'v1alpha1', 'ValidatingAdmissionPolicy'),
        plural='validatingadmissionpolicies',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class ValidatingAdmissionPolicy(res.GlobalResource, m_admissionregistration_v1alpha1.ValidatingAdmissionPolicy):
    """* **Extends**: ``models.admissionregistration_v1alpha1.ValidatingAdmissionPolicy``
       * **Type**: Global Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Status**: ``ValidatingAdmissionPolicyStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('admissionregistration.k8s.io', 'v1alpha1', 'ValidatingAdmissionPolicy'),
        plural='validatingadmissionpolicies',
        verbs=['delete', 'deletecollection', 'get', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Status: ClassVar = ValidatingAdmissionPolicyStatus


class ValidatingAdmissionPolicyBinding(res.GlobalResource, m_admissionregistration_v1alpha1.ValidatingAdmissionPolicyBinding):
    """* **Extends**: ``models.admissionregistration_v1alpha1.ValidatingAdmissionPolicyBinding``
       * **Type**: Global Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list`, `patch`, `create`, `replace`, `watch`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('admissionregistration.k8s.io', 'v1alpha1', 'ValidatingAdmissionPolicyBinding'),
        plural='validatingadmissionpolicybindings',
        verbs=['delete', 'deletecollection', 'get', 'list', 'patch', 'post', 'put', 'watch'],
    )

