# autogenerated module
from typing import ClassVar

from ..core import resource as res
from ..models import networking_v1alpha1 as m_networking_v1alpha1


class ClusterCIDR(res.GlobalResource, m_networking_v1alpha1.ClusterCIDR):
    """* **Extends**: ``models.networking_v1alpha1.ClusterCIDR``
       * **Type**: Global Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list`, `patch`, `create`, `replace`, `watch`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('networking.k8s.io', 'v1alpha1', 'ClusterCIDR'),
        plural='clustercidrs',
        verbs=['delete', 'deletecollection', 'get', 'list', 'patch', 'post', 'put', 'watch'],
    )


class IPAddress(res.GlobalResource, m_networking_v1alpha1.IPAddress):
    """* **Extends**: ``models.networking_v1alpha1.IPAddress``
       * **Type**: Global Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list`, `patch`, `create`, `replace`, `watch`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('networking.k8s.io', 'v1alpha1', 'IPAddress'),
        plural='ipaddresses',
        verbs=['delete', 'deletecollection', 'get', 'list', 'patch', 'post', 'put', 'watch'],
    )

